/////////////////////////////////////////////////////////////////////////////
//                                                                           
// Includes


#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/nodes/SoShapeHints.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoTexture2.h>
#include <Inventor/nodes/SoTexture2Transform.h>
#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/draggers/SoTranslate2Dragger.h>
#include <Inventor/nodes/SoPickStyle.h>
#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/nodes/SoSphere.h>

#include "grid.h"
#include "oscillator.h"
#include "lineSetToQuadMesh.h"

/////////////////////////////////////////////////////////////////////////////
//                                                                            
// Global Variables

extern OscillatorStruct Oscillator;
extern GridStruct       Grid;
SoSeparator *root;

void writeScene(){
  static int sceneNum = 0;
  char filename[50];
  SoOutput *output;
  SoWriteAction *wa;

  // write out scene
  output = new SoOutput();
  sprintf(filename, "scene%d.iv", sceneNum++);
  if(output->openFile(filename)){
    wa = new SoWriteAction(output);
    wa->apply(root);
    output->closeFile();
  }
  else {
    fprintf(stderr, "unknown file i/o error :(\n");
  }
}


// This key press code is from Josh Grant (http://www.cs.fsu.edu/~grant)
//======================================================================
void keyboardEvents (void *data, SoEventCallback *eventCB)
{
  const SoEvent *ev = eventCB->getEvent();

  // Write scene with the S key!
  if (SO_KEY_PRESS_EVENT(ev, S)){
    fprintf(stderr, "Writing scene.\n");
    writeScene();
  }

}

void positionCallback(void *, SoDragger *dragger){
  SbVec3f pos = ((SoTranslate2Dragger *)dragger)->translation.getValue();
  Oscillator.position = SbVec2f(pos[0], pos[1]);
}


/////////////////////////////////////////////////////////////////////////////
//                                                                            
//  WWWWW                                                                     
// WW  WW                         WW                                          
// WW                             WW                                          
// WW      WW WW   WWW    WWWW   WWWW   WWW                                   
// WW      WWW    WW WW      WW   WW   WW WW                                  
// WW      WW     WWWWW   WWWWW   WW   WWWWW                                  
// WW  WW  WW     WW     WW  WW   WW   WW                                     
//  WWWW   WW      WWWW   WWWWW   WW    WWWW                                  
//                                                                            
//                                                                            
//  WWWW                                                                      
// WW  WW                                                                     
// WWW                                                                        
//  WWWW    WWW    WWW   WWWW    WWW                                          
//    WWW  WW WW  WW WW  WW WW  WW WW                                         
//     WW  WW     WWWWW  WW WW  WWWWW                                         
// WW  WW  WW WW  WW     WW WW  WW                                            
//  WWWW    WWW    WWWW  WW WW   WWWW                                         
//                                                                            
//                                                                            
// Create a scene graph with draggers that control waves
// in a 2D mesh.

SoSeparator *
createScene()
  {
  //
  //                              root
  //                                |
  //    -----------------------------
  //    |    |   |        |     |
  // view  com  two      mesh  quad
  //xform  plex side     co    mesh
  //       ity           ord  
  //                   
  root = new SoSeparator();

  // view orientation
  // Rotate the scene so that the z-axis is nearly vertical,
  // toward the y-direction. Rotate about the x-axis.
  SoTransform *viewTransform = new SoTransform();
    viewTransform->rotation.setValue(SbRotation
      (SbVec3f (1.0, 0.0, 0.0),  -M_PI*0.30) );
    viewTransform->rotation.setValue(
      viewTransform->rotation.getValue()  *
      SbRotation(SbVec3f (0.0, 1.0, 0.0),   M_PI*0.10 ) 
      ); 
    root->addChild(viewTransform);


  // This keypress code is from Josh Grant
  SoEventCallback *cb = new SoEventCallback();
  cb->addEventCallback(SoKeyboardEvent::getClassTypeId(),
	  	       keyboardEvents, NULL);
  root->addChild(cb);

    
  // complexity
  SoComplexity *complexity = new SoComplexity();
    complexity->textureQuality = 1.0;
    complexity->value = 0.8;
  root->addChild(complexity);

  // light
  double cubeSize = 10.0;
  SoPointLight *roomLight = new SoPointLight();
  roomLight->location.setValue(SbVec3f(0.0, 0.0, cubeSize/2-0.05));
  roomLight->intensity.setValue(1.0);
  roomLight->color.setValue(SbColor(0.8, 0.7, 0.7));
  root->addChild( roomLight );


  // INVENTOR HACK: set up the hints to produce two-sided lighting
  SoShapeHints *twoSided = new SoShapeHints();
    twoSided->shapeType = SoShapeHints::UNKNOWN_SHAPE_TYPE;
    twoSided->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
  root->addChild(twoSided);


  // Cube Background stuff
  SoSeparator *cubeSep = new SoSeparator();
  SoPickStyle *unpickStyle = new SoPickStyle();
  unpickStyle->style.setValue(SoPickStyle::UNPICKABLE);
  cubeSep->addChild( unpickStyle );
  SoShapeHints *oneSided = new SoShapeHints();
  oneSided->shapeType = SoShapeHints::SOLID;
  oneSided->vertexOrdering = SoShapeHints::COUNTERCLOCKWISE;
  cubeSep->addChild(oneSided);
  SoTexture2Transform *tileTransform = new SoTexture2Transform();
  tileTransform->scaleFactor.setValue(SbVec2f(10.0, 10.0));
  cubeSep->addChild(tileTransform);
  SoTexture2 *tileTexture = new SoTexture2();
  tileTexture->filename.setValue("tile.rgb");
  cubeSep->addChild(tileTexture);
  SoCube *myCube = new SoCube();
  myCube->width.setValue(-cubeSize);
  myCube->depth.setValue(-cubeSize);
  myCube->height.setValue(-cubeSize);
  cubeSep->addChild(myCube);
  root->addChild(cubeSep);

  // invisible cube
  SoSeparator *invisSep = new SoSeparator();
  invisSep->addChild(unpickStyle);
  SoMaterial *invisMat = new SoMaterial();
  invisMat->transparency.setValue(1.0);
  invisSep->addChild(invisMat);
  SoCube *invisCube = new SoCube();
  invisCube->width.setValue(cubeSize);
  invisCube->height.setValue(cubeSize);
  invisCube->depth.setValue(cubeSize);
  invisSep->addChild(invisCube);
  root->addChild(invisSep);

  // floor
  SoSeparator *floorSep = new SoSeparator();
  SoTransform *floorTrans = new SoTransform();
  floorTrans->translation.setValue(SbVec3f(0.0, 0.0, -4.98));
  floorSep->addChild(floorTrans);
  SoTexture2Transform *floorTextTransform = new SoTexture2Transform();
  floorTextTransform->scaleFactor.setValue(SbVec2f(10.0, 10.0));
  //floorSep->addChild(floorTextTransform);
  SoTexture2 *floorTexture = new SoTexture2();
  floorTexture->filename.setValue("floor.rgb");
  floorSep->addChild(floorTexture);
  SoCube *floorCube = new SoCube();
  floorCube->width.setValue(cubeSize);
  floorCube->depth.setValue(0.02);
  floorCube->height.setValue(cubeSize);
  floorSep->addChild(floorCube);
  root->addChild(floorSep);
  
  // dragger
  SoSeparator *dragSep = new SoSeparator();
  SoTranslate2Dragger *faucDrag = new SoTranslate2Dragger();
  faucDrag->translation.setValue(SbVec3f(0.0, 0.0, 2.0));
  faucDrag->addMotionCallback(positionCallback, root);
  faucDrag->addStartCallback(positionCallback, root);
  dragSep->addChild(faucDrag);
  root->addChild(dragSep);

  // dragger appearance
  SoSeparator *dragAppSep = new SoSeparator();
  SoMaterial *dragAppMat = new SoMaterial();
  dragAppMat->diffuseColor.setValue(SbColor(0.8, 0.9, 0.9));
  dragAppSep->addChild(dragAppMat);
  SoTransform *dragAppTrans = new SoTransform();
  //dragAppTrans->translation.setValue(SbVec3f(0.0, 0.0, 2.0));
  dragAppTrans->scaleFactor.setValue(SbVec3f(8.0, 8.0, 8.0));
  dragAppTrans->rotation.setValue(SbRotation(SbVec3f(0.0, 0.393325, -0.9194), 3.0));
  dragAppSep->addChild(dragAppTrans);
  SoInput dragInput;
  dragInput.openFile("mug.iv");
  SoSeparator *dragAppModelSep = SoDB::readAll(&dragInput);
  dragAppSep->addChild(dragAppModelSep);
  dragAppSep->ref();
  faucDrag->setPart("translator", dragAppSep);

  // faucet 
  SoSeparator *fauceSep = new SoSeparator();
  SoCoordinate3 *fauceCoord = new SoCoordinate3();
  fauceCoord->point.set1Value(0, 0.0, 0.0, 0.0);
  fauceCoord->point.set1Value(1, 0.0, 0.0, 0.75);
  fauceCoord->point.set1Value(2, 0.0, 0.5, 1.5);
  fauceCoord->point.set1Value(3, 0.0, 1.0, 1.75);
  fauceCoord->point.set1Value(4, 0.0, 2.0, 1.825);
  fauceCoord->point.set1Value(5, 0.0, 3.0, 1.825);
  fauceCoord->point.set1Value(6, 0.0, 4.0, 1.825);
  //fauceCoord->point.set1Value(0, 0.0, 0.0, 0.0);
  fauceSep->addChild(fauceCoord);
  SoLineSet *fauceLineSet = new SoLineSet();
  fauceLineSet->numVertices.setValue(7);
  fauceLineSet->startIndex.setValue(0);
  //fauceLineSet->vertexProperty.setValue(fauceCoord);
  fauceSep->addChild(fauceLineSet);
  SoSeparator *fauceMeshSep = new SoSeparator();
  lineSetToQuadMesh(fauceMeshSep, fauceCoord, fauceLineSet, 0.5, 10);
  root->addChild(fauceMeshSep);

  // faucet knob
  SoSeparator *fauceKnobSep = new SoSeparator();
  SoTransform *knobCylXform = new SoTransform();
  knobCylXform->rotation.setValue(SbRotation(SbVec3f(1.0, 0.0, 0.0), SbVec3f(0.0, 0.0, 1.0)));
  knobCylXform->translation.setValue(SbVec3f(0.0, 0.0, 2.5));
  SoCylinder *knobCyl = new SoCylinder();
  knobCyl->radius.setValue(0.5);
  knobCyl->height.setValue(2.0);
  SoTransform *knobSphere1Xform = new SoTransform();
  knobSphere1Xform->translation.setValue(0.0, 0.0, 1.0);
  SoTransform *knobSphere2Xform = new SoTransform();
  knobSphere2Xform->translation.setValue(0.0, 0.0, -1.0);
  SoSphere *knobSphere = new SoSphere();
  knobSphere->radius.setValue(0.6);
  SoSeparator *knobSphere1Sep = new SoSeparator();
  knobSphere1Sep->addChild(knobSphere1Xform);
  knobSphere1Sep->addChild(knobSphere);
  SoSeparator *knobSphere2Sep = new SoSeparator();
  knobSphere2Sep->addChild(knobSphere2Xform);
  knobSphere2Sep->addChild(knobSphere);
  fauceKnobSep->addChild(knobCylXform);
  fauceKnobSep->addChild(knobCyl);
  fauceKnobSep->addChild(knobSphere1Sep);
  fauceKnobSep->addChild(knobSphere2Sep);
  SoTransform *knob2Xform = new SoTransform();
  knob2Xform->rotation.setValue(SbRotation(SbVec3f(1.0, 0.0, 0.0), SbVec3f(0.0, 1.0, 0.0)));
  fauceKnobSep->addChild(knob2Xform);
  fauceKnobSep->addChild(knobCyl);
  fauceKnobSep->addChild(knobSphere1Sep);
  fauceKnobSep->addChild(knobSphere2Sep);
  root->addChild(fauceKnobSep);
  

  // dragger active appearance
  SoSeparator *dragActAppSep = new SoSeparator();
  SoMaterial *dragActAppMat = new SoMaterial();
  dragActAppMat->diffuseColor.setValue(SbColor(1.0, 0.3, 0.3));
  dragActAppSep->addChild(dragActAppMat);
  dragActAppSep->addChild(dragAppTrans);
  dragActAppSep->addChild(dragAppModelSep);
  dragActAppSep->ref();
  faucDrag->setPart("translatorActive", dragActAppSep);

  // surface transform
  SoTransform *surfTrans = new SoTransform();
  surfTrans->translation.setValue(SbVec3f(-cubeSize/2.0, -cubeSize/2.0, 0.0));
  surfTrans->scaleFactor.setValue(SbVec3f(cubeSize, cubeSize, 1.0));
  root->addChild(surfTrans);

  // surface texture
  SoTexture2 *surfTexture = new SoTexture2();
  surfTexture->filename.setValue("water.rgb");
  root->addChild(surfTexture);

  // surface material
  SoMaterial *surfMat = new SoMaterial();
  surfMat->transparency.setValue(0.0);
  root->addChild(surfMat);

  // mesh coordinates
  SoCoordinate3 *coord = new SoCoordinate3();
  coord->point.setNum(Grid.sDim * Grid.tDim);
  root->addChild(coord);
  Grid.coord = coord;

  // initalize mesh x, y
  SbVec3f *vertexPositions = coord->point.startEditing();
  for (int ti = 0; ti < Grid.tDim; ti++)
    for (int si = 0; si < Grid.sDim; si++)
      vertexPositions[ti*Grid.sDim + si].setValue( si/(Grid.sDim-1.0),
						    ti/(Grid.tDim-1.0),
						    0.0);
  coord->point.finishEditing();

  // quadMesh
  SoQuadMesh *quadMesh = new SoQuadMesh;
    quadMesh->verticesPerRow    = Grid.sDim;
    quadMesh->verticesPerColumn = Grid.tDim;
  root->addChild(quadMesh);
  Grid.mesh = quadMesh;


  return root;
  }

