/////////////////////////////////////////////////////////////////////////////
//                                                                           
// Includes

#include <stdio.h>
#include <math.h>

#include <Xm/Form.h>
#include <Xm/MainW.h>
#include <Xm/Xm.h>

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>




/////////////////////////////////////////////////////////////////////////////
//                                                                            
// Global Variables











////////////////////////////////////////////////////////////
//
// Create and layout the main window.
//
// This section calls lots of X windows utilities.
// For details on the X library, the X toolkit, and
// the Motif look-and-feel, consult the multi-volume
// "Definitive Guides to the X Window System" published
// by O'Reilly. Frequently asked questions (FAQs) are
// available on the World Wide Web at
//
//    http://www.faqs.org/faqs/x-faq/
//    http://www.faqs.org/faqs/motif-faq/
//    
// Info is also available through Google Groups
//
//    http://groups.google.com/groups?group=comp.windows.x



SoXtExaminerViewer *
createAndLayoutMainWindow(int argc, char **argv)
  {
  XtAppContext  appContext;
  Widget        topLevelXShell;

  // Use the Xt/Motif internationalization and localization rules.
  XtSetLanguageProc (NULL, NULL, NULL);

  // The top-level shell communicates between the
  // application and the window manager running the desktop.
  // We create it using xt (X toolkit).
  //
  // X-style arguments are parsed and removed from the
  // argument list. For example
  //
  //  -geometry 800x600+0+0   
  //
  // creates an 800 by 600 pixel window, placed at the 
  // upper left corner of the screen.

  topLevelXShell = XtVaAppInitialize(
    &appContext,        // context that will be returned
    "Ivview",           // application class
    NULL, 0,            // options
    &argc, argv,        // command line args in/out
    NULL,               // fallback_resources
    NULL, 0);           // args, num_args


  Widget main_w = XtVaCreateManagedWidget ("main_window", 
    xmMainWindowWidgetClass,     topLevelXShell, 
    NULL);

    
  // Create a "form" as the top-level shell's only child.
  // A form is a stretchable container for other X window
  // components. In this case, it will contain the Inventor
  // examiner viewer.
  // 
  // Created using xt (X toolkit).

  Widget form = XtCreateWidget("Form", 
    xmFormWidgetClass,    main_w, 
    NULL, 0);

  XtVaSetValues(form,
    XmNwidth,  800,
    XmNheight, 600,
    NULL);


  // Initialize Inventor inside the form
  SoXt::init(form);

  // Create a viewer inside the form
  SoXtExaminerViewer *viewer = new SoXtExaminerViewer(form);

  // Attach the viewer to the edges of the form.
  XtVaSetValues(viewer->getWidget(), 
    XmNtopAttachment,    XmATTACH_FORM, 
    XmNbottomAttachment, XmATTACH_FORM,
    XmNleftAttachment,   XmATTACH_FORM, 
    XmNrightAttachment,  XmATTACH_FORM,
    NULL);

  // Let the form handle events.
  XtManageChild(form);


  // Now that the descendents of the top-level shell
  // all have their states finalized, the top-level shell
  // can be instantiated.
  XtRealizeWidget(topLevelXShell);



  return (viewer);
  }

